#!/usr/bin/python3
dbuser = "steemax"
dbpass = "SteemAX_pass23"
dbname = "steemax"
msgmode = "verbose"
logpath = ""
logfilename = "steemax.log"
webpath = ""
recaptcha_url = "https://www.google.com/recaptcha/api/siteverify"
recaptcha_secret = ""
client_id = ""
client_secret = ""
callback_url = ""
